% wfgextend.M

% NOTE: wfgextend function is developed based on the original wfg function 
% wfgextend includes all the nine wfg functions, in addition, more test 
% functions (i.e., WFG41 to WFG48) with different Pareto front (PF) shapes 
% are generated 

% wfgextend is an extended version of WFG test problems. 
% In addition to the original wfg (wfg1 to wfg9) problems. The wfgextend 
% introduces two sets of new problems with various PF geometries. 
% WFG14-WFG19 and WFG41-WFG48

% WFG14-WFG19: are created by applying h_convex function to WFG4 to WFG9, respectively
% WFG41-WFG48: are created by applying different shape functions provided in the WFG suite to the original WFG4 problem.

%
% Scalable test problem toolkit.
%
% F = wfgextend(Z, M, k, l, testNo,S_bounds)
%
% Inputs: Z      - candidate solutions  [0 0 0 ... 0; 2 4 6 8 .. 2n]
%         M      - no. of objectives
%         k      - no. of position-related parameters
%         l      - no. of distance-related parameters
%         testNo - test problem no.    Three sets of testNo: 1: testNo = {1 2 3 4 5 6 7 8 9}
%                                                            2: testNo = {14 15 16 17 18 19}
%                                                            3: testNo = {41 42 43 44 45 46 47 48}
%         
%        
%         S_bounds - set the upper bounds of the PF range, e.g., [2 ,4] when M=2
%
% Output: F      - objective vectors
%
%
%% Detail about WFG1 to WFG9  (More details are in Ref[1])
% These are identical to the original problems

%% Detail about WFG41 to WFG48
% wfg14 is WFG4 with h_convex shape;
% wfg15 is WFG5 with h_convex shape;
% wfg16 is WFG6 with h_convex shape;
% wfg17 is WFG7 with h_convex shape;
% wfg18 is WFG8 with h_convex shape;
% wfg19 is WFG9 with h_convex shape;

%% Detail about WFG41 to WFG48 (More details are in Ref[2])
% WFG41 is the same as the WFG4 problem which has a concave PF.
% WFG42 has a convex PF. It is built by replacing the concave shape function used in WFG4 with the convex shape function.
% WFG43 has a strong concave PF. It is built by scaling the concave shape function with power $\frac{1}{4}$.
% WFG44 has a strong convex PF. It is built by scaling the convex shape function with power $\frac{1}{4}$.
% WFG45 has a mixed PF. It is built by replacing the concave shape function used in WFG4 with the mixed shape function.
% The PF of WFG46 is a hyperplane. It is built by replacing the concave shape function used in WFG4 with the linear shape function.
% The PF of WFG47 is disconnected and concave. It is built by replacing the concave shape function used in WFG4 with the concave (for the first $M-1$ objectives) and the disconnected (for the last objective) shape function. Parameters used in the disconnected function are set as $\alpha=\beta=\frac{1}{2}, A = 2$.
% The PF of WFG48 is disconnected and convex. It is built by replacing the concave shape function used in WFG4 with the convex (for the first $M-1$ objectives) and the disconnected (for the last objective) shape function.  Parameters used in the disconnected function are set as $\alpha=\beta=\frac{1}{2}, A = 2$.

%% Please cite the following papers when you are using this code
% Ref[1]: Huband S, Hingston P, Barone L, While L, 2006, A review
%      of multiobjective test problems and a scalable test problem
%      toolkit. IEEE Transactions on Evolutionary Computation,
%      10(5), pp477-506.
% Ref[2]: Wang, R., Purshouse, R. C., Fleming, P. J., Preference-inspired 
%      co-evolutionary algorithms using weight vectors, European Journal
%      of Operational Research, 243(2), 423-441, 2015.
% Ref[3]: Wang, R., Zhang, Q. F., Zhang, T., Decomposition based algorithms 
%      using Pareto adaptive scalarizing methods IEEE Transactions on 
%      Evolutionary Computation, IEEE, 2015, To appear

%% Robin Purshouse, 27 April 2010  Created
% Rui Wang, 11 DEC 2011  Updated
% Rui Wang, 15 OCT 2015  Updated
% email: ruiwangnudt@gmail.com  

%%
function F = wfgextend(Z, M, k, l, testNo,S_bounds)

% Check for correct number of inputs.
if nargin < 6
    % Could also check data input range z_i in [0, 2i].
    S = NaN * ones(1, M);
    for i = 1:M
        S(i) = 2*i;
    end
    S_bounds = S;
end
if size(S_bounds,2)~=M
    error('Inconsistent: S_bounds with M.');
end
% Get total number of decision variables and no. of candidates.
[noSols, n] = size(Z);

% Data input checks.
if n ~= (k + l)
    error('Inconsistent number of variabes.');
end
if rem(k,M-1) ~= 0
    error('k must be divisible by M-1.');
end
if (testNo == 2 || testNo == 3) && (rem(l,2) ~= 0)
    error('l must be a multiple of 2 for WFG2 and WFG3.');
end
% Initialise function-wide constants.
NO_TESTS = 48;
S = S_bounds;

D = 1;
A = ones(NO_TESTS, M-1);
A(3,2:M-1) = 0;

% Transform all variable ranges to [0 1].
Y = Z;
for i = 1:n
    Y(:,i) = Z(:,i) ./ (2*i);
end

if testNo == 1

    % Apply first transformation.
    Ybar = Y;
    lLoop = k + 1;
    shiftA = 0.35;
    Ybar(:, lLoop:n) = s_linear(Ybar(:, lLoop:n), shiftA);

    % Apply second transformation.
    Ybarbar = Ybar;
    biasA = 0.8;
    biasB = 0.75;
    biasC = 0.85;
    Ybarbar(:, lLoop:n) = b_flat(Ybarbar(:, lLoop:n), biasA, biasB, ...
        biasC);

    % Apply third transformation.
    Ybarbarbar = Ybarbar;
    biasA = 0.02;
    Ybarbarbar = b_poly(Ybarbarbar, biasA);

    % Apply fourth transformation.
    T = NaN * ones(noSols, M);
    uLoop = M - 1;
    for i = 1:uLoop
        lBnd = 1+(i-1)*k/(M-1);
        uBnd = i*k/(M-1);
        weights = 2*(lBnd:uBnd);
        T(:,i) = r_sum(Ybarbarbar(:,lBnd:uBnd), weights);
    end
    T(:,M) = r_sum(Ybarbarbar(:,lLoop:n), 2*(lLoop:n));

    % Apply degeneracy constants.
    X = T;
    for i = 1:M-1
        X(:,i) = max(T(:,M), A(testNo, i)) .* (T(:,i) - 0.5) + 0.5;
    end

    % Generate objective values.
    fM = h_mixed(X(:,1), 1, 5);
    F = h_convex(X(:,1:uLoop));
    F(:,M) = fM;
    F = rep(D*X(:,M), [1,M]) + rep(S, [noSols 1]) .* F;

elseif testNo == 2 || testNo == 3

    % Apply first transformation.
    Ybar = Y;
    lLoop = k + 1;
    shiftA = 0.35;
    Ybar(:, lLoop:n) = s_linear(Ybar(:, lLoop:n), shiftA);

    % Apply second transformation.
    Ybarbar = Ybar;
    uLoop = k + l/2;
    for i = lLoop:uLoop
        lBnd = k+2*(i-k)-1;
        uBnd = k+2*(i-k);
        Ybarbar(:,i) = r_nonsep(Ybar(:,lBnd:uBnd), 2);
    end

    % Apply third transformation.
    T = NaN * ones(noSols, M);
    uLoop = M - 1;
    weights = ones(1, k/(M-1));
    for i = 1:uLoop
        lBnd = 1+(i-1)*k/(M-1);
        uBnd = i*k/(M-1);
        T(:,i) = r_sum(Ybarbar(:,lBnd:uBnd), weights);
    end
    T(:,M) = r_sum(Ybarbar(:,lLoop:k+l/2), ones(1, (k+l/2)-lLoop+1));

    % Apply degeneracy constants.
    X = T;
    for i = 1:M-1
        X(:,i) = max(T(:,M), A(testNo, i)) .* (T(:,i) - 0.5) + 0.5;
    end

    % Generate objective values.
    if testNo == 2
        fM = h_disc(X(:,1), 1, 1, 5);
        F = h_convex(X(:,1:uLoop));
        F(:,M) = fM;
        F = rep(D*X(:,M), [1,M]) + rep(S, [noSols 1]) .* F;
    else
        F = h_linear(X(:,1:uLoop));
        F = rep(D*X(:,M), [1,M]) + rep(S, [noSols 1]) .* F;
    end

elseif testNo == 4 || testNo==41 || testNo==42 ||  testNo == 43 || testNo == 44 || testNo == 45 || testNo == 46 || testNo == 47 || testNo == 48 || testNo == 49  || testNo==14 || testNo==5 || testNo==15

    % Apply first transformation.
    if testNo == 4 || testNo==14 || testNo == 41 || testNo == 42 || testNo == 43 || testNo == 44 || testNo == 45 || testNo == 46  || testNo==47 || testNo==48 
        shiftA = 30;
        shiftB = 10;
        shiftC = 0.35;
        Ybar = s_multi(Y, shiftA, shiftB, shiftC);
    elseif testNo == 5 || testNo==15
        shiftA = 0.35;
        shiftB = 0.001;
        shiftC = 0.05;
        Ybar = s_decep(Y, shiftA, shiftB, shiftC);
    end
    % Apply second transformation.
    T = NaN * ones(noSols, M);
    lLoop = k + 1;
    uLoop = M - 1;
    weights = ones(1, k/(M-1));
    for i = 1:uLoop
        lBnd = 1+(i-1)*k/(M-1);
        uBnd = i*k/(M-1);
        T(:,i) = r_sum(Ybar(:,lBnd:uBnd), weights);
    end
    T(:,M) = r_sum(Ybar(:,lLoop:n), ones(1, n-lLoop+1));

    % Apply degeneracy constants.
    X = T;
    for i = 1:M-1
        X(:,i) = max(T(:,M), A(testNo, i)) .* (T(:,i) - 0.5) + 0.5;
    end
    % Generate objective values.
    if  testNo == 4 || testNo == 5
        F = rep(D*X(:,M), [1,M]) + rep(S, [noSols 1]) .* h_concave(X(:,1:uLoop));
    elseif testNo == 14 || testNo == 15
        F = rep(D*X(:,M), [1,M]) + rep(S, [noSols 1]) .* h_convex(X(:,1:uLoop));
    elseif testNo==41
        F = rep(D*X(:,M), [1,M]) + rep(S, [noSols 1]) .* h_concave(X(:,1:uLoop));
    elseif testNo==42
        F = rep(D*X(:,M), [1,M]) + rep(S, [noSols 1]) .* h_convex(X(:,1:uLoop));
    elseif testNo==43
        F = rep(D*X(:,M), [1,M]) + rep(S, [noSols 1]) .* h_concave_sharp(X(:,1:uLoop));
    elseif testNo==44
        F = rep(D*X(:,M), [1,M]) + rep(S, [noSols 1]) .* h_convex_sharp(X(:,1:uLoop));
    elseif testNo == 45
        fM = h_mixed(X(:,1), 1, 2);
        F = h_concave(X(:,1:uLoop));
        F(:,M) = fM;
        F = rep(D*X(:,M), [1,M]) + rep(S, [noSols 1]) .* F;
    elseif testNo==46
        F = rep(D*X(:,M), [1,M]) + rep(S, [noSols 1]) .* h_linear(X(:,1:uLoop));
    elseif testNo == 47
        fM = h_disc22(X(:,1), 1/2, 1/2, 2);
        F = h_concave(X(:,1:uLoop));
        F(:,M) = fM;
        F = rep(D*X(:,M), [1,M]) + rep(S, [noSols 1]) .* F;
    elseif testNo==48
        %F = rep(D*X(:,M), [1,M]) + rep(S, [noSols 1]) .*  h_disc(X(:,1:uLoop));
        %fM = h_disc(X(:,1), 1, 1, 2);
        fM = h_disc22(X(:,1), 1/2, 1/2, 2);
        F = h_convex(X(:,1:uLoop));
        F(:,M) = fM;
        F = rep(D*X(:,M), [1,M]) + rep(S, [noSols 1]) .* F;
    
    end
elseif testNo == 11 || testNo == 12 || testNo == 13
    % all the problems are re-produced by WFG4;
    % Apply first transformation.
    shiftA = 30;
    shiftB = 10;
    shiftC = 0.35;
    Ybar = s_multi(Y, shiftA, shiftB, shiftC);

    % Apply second transformation.
    T = NaN * ones(noSols, M);
    lLoop = k + 1;
    uLoop = M - 1;
    weights = ones(1, k/(M-1));
    for i = 1:uLoop
        lBnd = 1+(i-1)*k/(M-1);
        uBnd = i*k/(M-1);
        T(:,i) = r_sum(Ybar(:,lBnd:uBnd), weights);
    end
    T(:,M) = r_sum(Ybar(:,lLoop:n), ones(1, n-lLoop+1));

    % Apply degeneracy constants.
    X = T;
    for i = 1:M-1
        X(:,i) = max(T(:,M), A(testNo, i)) .* (T(:,i) - 0.5) + 0.5;
    end
    
    % Generate objective values.
    if testNo == 12
        F = rep(D*X(:,M), [1,M]) + rep(S, [noSols 1]) .* h_concave_sharp(X(:,1:uLoop));
    elseif testNo == 13
        F = rep(D*X(:,M), [1,M]) + rep(S, [noSols 1]) .* h_convex_sharp(X(:,1:uLoop));
    elseif testNo==11
        % Generate objective values.
        fM = h_mixed(X(:,1), 1, 3);
        %F = h_concave(X(:,1:uLoop));
        F = h_convex(X(:,1:uLoop));
        F(:,M) = fM;
        F = rep(D*X(:,M), [1,M]) + rep(S, [noSols 1]) .* F;
    end
elseif testNo == 6 || testNo == 16
    % Apply first transformation.
    Ybar = Y;
    lLoop = k + 1;
    shiftA = 0.35;
    Ybar(:, lLoop:n) = s_linear(Ybar(:, lLoop:n), shiftA);

    % Apply second transformation.
    T = NaN * ones(noSols, M);
    uLoop = M - 1;
    for i = 1:uLoop
        lBnd = 1+(i-1)*k/(M-1);
        uBnd = i*k/(M-1);
        T(:,i) = r_nonsep(Ybar(:,lBnd:uBnd), k/(M-1));
    end
    T(:,M) = r_nonsep(Ybar(:,k+1:k+l), l);

    % Apply degeneracy constants.
    X = T;
    for i = 1:M-1
        X(:,i) = max(T(:,M), A(testNo, i)) .* (T(:,i) - 0.5) + 0.5;
    end

    % Generate objective values.
    if  testNo == 6
        F = rep(D*X(:,M), [1,M]) + rep(S, [noSols 1]) .* h_concave(X(:,1:uLoop));
    elseif testNo == 16
        F = rep(D*X(:,M), [1,M]) + rep(S, [noSols 1]) .* h_convex(X(:,1:uLoop));
    end

elseif testNo == 7 || testNo == 17

    % Apply first transformation.
    Ybar = Y;
    biasA = 0.98 / 49.98;
    biasB = 0.02;
    biasC = 50;
    for i = 1:k
        Ybar(:,i) = b_param(Y(:,i), r_sum(Y(:,i+1:n), ones(1, n-i)), ...
            biasA, biasB, biasC);
    end

    % Apply second transformation.
    Ybarbar = Ybar;
    lLoop = k + 1;
    shiftA = 0.35;
    Ybarbar(:, lLoop:n) = s_linear(Ybar(:, lLoop:n), shiftA);

    % Apply third transformation.
    T = NaN * ones(noSols, M);
    lLoop = k + 1;
    uLoop = M - 1;
    weights = ones(1, k/(M-1));
    for i = 1:uLoop
        lBnd = 1+(i-1)*k/(M-1);
        uBnd = i*k/(M-1);
        T(:,i) = r_sum(Ybarbar(:,lBnd:uBnd), weights);
    end
    T(:,M) = r_sum(Ybarbar(:,lLoop:n), ones(1, n-lLoop+1));

    % Apply degeneracy constants.
    X = T;
    for i = 1:M-1
        X(:,i) = max(T(:,M), A(testNo, i)) .* (T(:,i) - 0.5) + 0.5;
    end

    % Generate objective values.
    if  testNo == 7
        F = rep(D*X(:,M), [1,M]) + rep(S, [noSols 1]) .* h_concave(X(:,1:uLoop));
    elseif testNo == 17
        F = rep(D*X(:,M), [1,M]) + rep(S, [noSols 1]) .* h_convex(X(:,1:uLoop));
    end
elseif testNo == 8 || testNo == 18 || testNo == 28

    % Apply first transformation.
    Ybar = Y;
    lLoop = k + 1;
    biasA = 0.98 / 49.98;
    biasB = 0.02;
    biasC = 50;
    for i = lLoop:n
        Ybar(:,i) = b_param(Y(:,i), r_sum(Y(:,1:i-1), ones(1, i-1)), ...
            biasA, biasB, biasC);
    end

    % Apply second transformation.
    Ybarbar = Ybar;
    shiftA = 0.35;
    Ybarbar(:, lLoop:n) = s_linear(Ybar(:, lLoop:n), shiftA);

    % Apply third transformation.
    T = NaN * ones(noSols, M);
    lLoop = k + 1;
    uLoop = M - 1;
    weights = ones(1, k/(M-1));
    for i = 1:uLoop
        lBnd = 1+(i-1)*k/(M-1);
        uBnd = i*k/(M-1);
        T(:,i) = r_sum(Ybarbar(:,lBnd:uBnd), weights);
    end
    T(:,M) = r_sum(Ybarbar(:,lLoop:n), ones(1, n-lLoop+1));

    % Apply degeneracy constants.
    X = T;
    for i = 1:M-1
        X(:,i) = max(T(:,M), A(testNo, i)) .* (T(:,i) - 0.5) + 0.5;
    end
    % Generate objective values.
    if  testNo == 8
        F = rep(D*X(:,M), [1,M]) + rep(S, [noSols 1]) .* h_concave(X(:,1:uLoop));
    elseif testNo==18
        F = rep(D*X(:,M), [1,M]) + rep(S, [noSols 1]) .* h_convex(X(:,1:uLoop));
    end
elseif testNo == 9 || testNo == 19 || testNo==29

    % Apply first transformation.
    Ybar = Y;
    uLoop = n - 1;
    biasA = 0.98 / 49.98;
    biasB = 0.02;
    biasC = 50;
    for i = 1:uLoop
        Ybar(:,i) = b_param(Y(:,i), r_sum(Y(:,i+1:n), ones(1, n-i)), ...
            biasA, biasB, biasC);
    end

    % Apply second transformation.
    Ybarbar = Ybar;
    biasA = 0.35;
    biasB = 0.001;
    biasC = 0.05;
    Ybarbar(:,1:k) = s_decep(Ybar(:,1:k), biasA, biasB, biasC);
    biasA = 30;
    biasB = 95;
    biasC = 0.35;
    Ybarbar(:,k+1:n) = s_multi(Ybar(:,k+1:n), biasA, biasB, biasC);

    % Apply third transformation.
    T = NaN * ones(noSols, M);
    uLoop = M - 1;
    for i = 1:uLoop
        lBnd = 1+(i-1)*k/(M-1);
        uBnd = i*k/(M-1);
        T(:,i) = r_nonsep(Ybarbar(:,lBnd:uBnd), k/(M-1));
    end
    T(:,M) = r_nonsep(Ybarbar(:,k+1:k+l), l);

    % Apply degeneracy constants.
    X = T;
    for i = 1:M-1
        X(:,i) = max(T(:,M), A(testNo, i)) .* (T(:,i) - 0.5) + 0.5;
    end

    % Generate objective values.
    if  testNo == 9
        F = rep(D*X(:,M), [1,M]) + rep(S, [noSols 1]) .* h_concave(X(:,1:uLoop));
    elseif testNo==19
        F = rep(D*X(:,M), [1,M]) + rep(S, [noSols 1]) .* h_convex(X(:,1:uLoop));
    end
    

else
    error('Requested test function not implemented.');
end
end

% Transformation functions.

% Reduction: weighted sum.
function ybar = r_sum(y, weights)

[noSols noY] = size(y);
wgtMatrix=rep(weights,[noSols 1]);
ybar = y .* wgtMatrix;
ybar = sum(ybar, 2) ./ sum(wgtMatrix, 2);

end

% Reduction: non-separable.
function y_bar = r_nonsep(y, A)

[noSols noY] = size(y);

y_bar = 0;
for j = 1:noY
    innerSum = 0;
    for k = 0:(A-2)
        innerSum = innerSum + abs(y(:,j) - y(:,1+mod(j+k,noY)));
    end
    y_bar = y_bar + y(:,j) + innerSum;
end
y_bar = y_bar / ( (noY/A) * ceil(A/2) * (1+2*A-2*ceil(A/2)) );

end

% Bias: polynomial.
function y_bar = b_poly(y, alpha)

y_bar = y.^alpha;

end

% Bias: flat region
function y_bar = b_flat(y, A, B, C)

[noSols noY] = size(y);
min1 = min(0, floor(y - B));
min2 = min(0, floor(C - y));
y_bar = A + min1*A.*(B-y)/B - min2*(1-A).*(y-C)/(1-C);

% Machine precision problems can cause y_bar to go slightly negative so
% force >=0 condition.
y_bar=max(0,y_bar);

end

% Bias: parameter dependent.
function ybar = b_param(y, uy, A, B, C)

[noSols noY] = size(y);
v = A - (1 - 2*uy) .* abs(floor(0.5 - uy) + A);
v = rep(v, [1 noY]);
ybar = y.^(B + (C-B)*v);

end

% Shift: linear.
function ybar = s_linear(y, A)

ybar = abs(y - A) ./ abs(floor(A - y) + A);

end

% Shift: deceptive.
function ybar = s_decep(y, A, B, C)

y1 = floor(y - A + B) * (1 - C + (A - B)/B) / (A - B);
y2 = floor(A + B - y) * (1 - C + (1 - A - B)/B) / (1 - A - B);
ybar = 1 + (abs(y - A) - B) .* (y1 + y2 + 1/B);

end

% Shift: multi-modal.
function ybar = s_multi(y, A, B, C)

y1 = abs(y-C) ./ (2*(floor(C-y)+C));
ybar = (1 + cos((4*A+2)*pi*(0.5 - y1)) + 4*B*y1.^2) / (B+2);

end

% Shape functions.
% Linear.
function f = h_linear(x)

[noSols mMinusOne] = size(x);

M = mMinusOne + 1;
f = NaN * ones(noSols, M);

f(:,1) = prod(x, 2);
for i = 2:mMinusOne
    f(:,i) = prod(x(:,1:M-i), 2) .* (1 - x(:,M-i+1));
end
f(:,M) = 1 - x(:,1);
end

% Convex.
function f = h_convex(x)
[noSols mMinusOne] = size(x);

M = mMinusOne + 1;
f = NaN * ones(noSols, M);

f(:,1) = prod(1-cos(x*pi/2), 2);
for i = 2:mMinusOne
    f(:,i) = prod(1-cos(x(:,1:M-i)*pi/2), 2) .* (1-sin(x(:,M-i+1)*pi/2));
end
f(:,M) = 1-sin(x(:,1)*pi/2);
end

% Concave.
function f = h_concave(x)

[noSols mMinusOne] = size(x);

M = mMinusOne + 1;
f = NaN * ones(noSols, M);

f(:,1) = prod(sin(x*pi/2), 2);
for i = 2:mMinusOne
    f(:,i) = prod(sin(x(:,1:M-i)*pi/2), 2) .* cos(x(:,M-i+1)*pi/2);
end
f(:,M) = cos(x(:,1)*pi/2);

end

function f = h_concave_sharp(x)
[noSols mMinusOne] = size(x);
M = mMinusOne + 1;
f = NaN * ones(noSols, M);
wRate = 1/4;
f(:,1) = prod(sin(x*pi/2), 2);
for i = 2:mMinusOne
    f(:,i) = prod( sin( x(:,1:M-i)*pi/2), 2 ) .* cos(x(:,M-i+1)*pi/2).^wRate;
end
f(:,M) = cos(x(:,1)*pi/2).^wRate;
end

% Added by Rui: to generate a sharp shape
% Concave 21/12/2011.
function f = h_convex_sharp(x)
[noSols mMinusOne] = size(x);
M = mMinusOne + 1;
f = NaN * ones(noSols, M);
wRate = 1/4;
f(:,1) = prod(1-cos(x*pi/2), 2);
for i = 2:mMinusOne
    f(:,i) = prod(1-cos(x(:,1:M-i)*pi/2), 2) .* (1-sin(x(:,M-i+1)*pi/2).^wRate);
end
f(:,M) = (1-sin(x(:,1)*pi/2).^wRate);
end



% Mixed.
function f = h_mixed(x,alpha,A)

f = (1 - x(:,1) - cos(2*A*pi*x(:,1) + pi/2) / (2*A*pi)).^alpha;

end

% Disconnected.
function f = h_disc22(x,alpha,beta,A)
f = 1 - x(:,1).^alpha .* cos(A * x(:,1).^(beta) * pi).^2;

end
% Disconnected.
function f = h_disc(x,alpha,beta,A)
f = 1 - x(:,1).^alpha .* cos(A * x(:,1).^beta * pi).^2;
end
